# fuck magic number

ROOM_EXPIRE_TIME = 30 * 60  # 30 minutes
ROOM_TOKEN_EXPIRE_TIME = 30 * 60  # 30 minutes
RANDOM_ROOM_EXPIRE_TIME = 5 * 60  # 5 minutes
MAX_RANDOM_ROOM_EXPIRE_TIME = 30 * 60  # 30 minutes
RANDOM_ROOM_TOKEN_EXPIRE_TIME = 5 * 60  # 5 minutes
DAILYLANGCHAINRAGBOT_EXPIRE_TIME = 25 * 60
